@@ -3,25 +3,9 @@
 // Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
 // Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
 //
-// This file is part of the Boost Graph Library
-//
-// You should have received a copy of the License Agreement for the
-// Boost Graph Library along with the software; see the file LICENSE.
-// If not, contact Office of Research, University of Notre Dame, Notre
-// Dame, IN 46556.
-//
-// Permission to modify the code and to distribute modified code is
-// granted, provided the text of this NOTICE is retained, a notice that
-// the code was modified is included with the above COPYRIGHT NOTICE and
-// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
-// file is distributed with the modified code.
-//
-// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
-// By way of example, but not limitation, Licensor MAKES NO
-// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
-// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
-// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
-// OR OTHER RIGHTS.
+// Distributed under the Boost Software License, Version 1.0. (See
+// accompanying file LICENSE_1_0.txt or copy at
+// http://www.boost.org/LICENSE_1_0.txt)
 //=======================================================================
 //
 #ifndef BOOST_GRAPH_CONCEPTS_HPP
@@ -34,126 +18,11 @@
 #include <boost/concept_check.hpp>
 #include <boost/detail/workaround.hpp>
 
-namespace boost {
+#include <boost/concept/detail/concept_def.hpp>
 
-  template <class T>
-  struct MultiPassInputIteratorConcept {
-    void constraints() {
-      function_requires< InputIteratorConcept<T> >();
-    }
-  };
-
-  template <class G>
-  struct GraphConcept
-  {
-    typedef typename graph_traits<G>::vertex_descriptor vertex_descriptor;
-    typedef typename graph_traits<G>::directed_category directed_category;
-    typedef typename graph_traits<G>::edge_parallel_category
-      edge_parallel_category;
-    typedef typename graph_traits<G>::traversal_category
-      traversal_category;
-    void constraints() {
-      function_requires< DefaultConstructibleConcept<vertex_descriptor> >();
-      function_requires< EqualityComparableConcept<vertex_descriptor> >();
-      function_requires< AssignableConcept<vertex_descriptor> >();
-    }
-    G g;
-  };
-
-  template <class G>
-  struct IncidenceGraphConcept
-  {
-    typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
-    typedef typename graph_traits<G>::out_edge_iterator
-      out_edge_iterator;
-    typedef typename graph_traits<G>::traversal_category
-      traversal_category;
-    void constraints() {
-      function_requires< GraphConcept<G> >();
-      function_requires< MultiPassInputIteratorConcept<out_edge_iterator> >();
-      function_requires< DefaultConstructibleConcept<edge_descriptor> >();
-      function_requires< EqualityComparableConcept<edge_descriptor> >();
-      function_requires< AssignableConcept<edge_descriptor> >();
-      function_requires< ConvertibleConcept<traversal_category,
-        incidence_graph_tag> >();
-
-      p = out_edges(v, g);
-      n = out_degree(v, g);
-      e = *p.first;
-      u = source(e, g);
-      v = target(e, g);
-      const_constraints(g);
-    }
-    void const_constraints(const G& g) {
-      p = out_edges(v, g);
-      n = out_degree(v, g);
-      e = *p.first;
-      u = source(e, g);
-      v = target(e, g);
-    }
-    std::pair<out_edge_iterator, out_edge_iterator> p;
-    typename graph_traits<G>::vertex_descriptor u, v;
-    typename graph_traits<G>::edge_descriptor e;
-    typename graph_traits<G>::degree_size_type n;
-    G g;
-  };
-
-  template <class G>
-  struct BidirectionalGraphConcept
-  {
-    typedef typename graph_traits<G>::in_edge_iterator
-      in_edge_iterator;
-    typedef typename graph_traits<G>::traversal_category
-      traversal_category;
-    void constraints() {
-      function_requires< IncidenceGraphConcept<G> >();
-      function_requires< MultiPassInputIteratorConcept<in_edge_iterator> >();
-      function_requires< ConvertibleConcept<traversal_category,
-        bidirectional_graph_tag> >();
-
-      p = in_edges(v, g);
-      n = in_degree(v, g);
-      e = *p.first;
-      const_constraints(g);
-    }
-    void const_constraints(const G& g) {
-      p = in_edges(v, g);
-      n = in_degree(v, g);
-      e = *p.first;
-    }
-    std::pair<in_edge_iterator, in_edge_iterator> p;
-    typename graph_traits<G>::vertex_descriptor v;
-    typename graph_traits<G>::edge_descriptor e;
-    typename graph_traits<G>::degree_size_type n;
-    G g;
-  };
-
-  template <class G>
-  struct AdjacencyGraphConcept
-  {
-    typedef typename graph_traits<G>::adjacency_iterator
-      adjacency_iterator;
-    typedef typename graph_traits<G>::traversal_category
-      traversal_category;
-    void constraints() {
-      function_requires< GraphConcept<G> >();
-      function_requires< MultiPassInputIteratorConcept<adjacency_iterator> >();
-      function_requires< ConvertibleConcept<traversal_category,
-        adjacency_graph_tag> >();
-
-      p = adjacent_vertices(v, g);
-      v = *p.first;
-      const_constraints(g);
-    }
-    void const_constraints(const G& g) {
-      p = adjacent_vertices(v, g);
-    }
-    std::pair<adjacency_iterator,adjacency_iterator> p;
-    typename graph_traits<G>::vertex_descriptor v;
-    G g;
-  };
-
-// dwa 2003/7/11 -- This clearly shouldn't be neccessary, but if
+namespace boost
+{
+// dwa 2003/7/11 -- This clearly shouldn't be necessary, but if
 // you want to use vector_as_graph, it is!  I'm sure the graph
 // library leaves these out all over the place.  Probably a
 // redesign involving specializing a template with a static
@@ -172,21 +41,141 @@ template <class T>
 typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
 #endif      
 
-  template <class G>
-  struct VertexListGraphConcept
+  namespace concepts {
+  BOOST_concept(MultiPassInputIterator,(T)) {
+    BOOST_CONCEPT_USAGE(MultiPassInputIterator) {
+        BOOST_CONCEPT_ASSERT((InputIterator<T>));
+    }
+  };
+
+  BOOST_concept(Graph,(G))
+  {
+    typedef typename graph_traits<G>::vertex_descriptor vertex_descriptor;
+    typedef typename graph_traits<G>::directed_category directed_category;
+    typedef typename graph_traits<G>::edge_parallel_category
+      edge_parallel_category;
+      
+      typedef typename graph_traits<G>::traversal_category
+      traversal_category;
+   
+      BOOST_CONCEPT_USAGE(Graph)
+      {
+          BOOST_CONCEPT_ASSERT((DefaultConstructible<vertex_descriptor>));
+          BOOST_CONCEPT_ASSERT((EqualityComparable<vertex_descriptor>));
+          BOOST_CONCEPT_ASSERT((Assignable<vertex_descriptor>));
+      }
+      G g;      
+  };
+
+  BOOST_concept(IncidenceGraph,(G))
+    : Graph<G>
+  {
+      typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
+      typedef typename graph_traits<G>::out_edge_iterator
+      out_edge_iterator;
+      
+      typedef typename graph_traits<G>::traversal_category
+      traversal_category;
+      
+      BOOST_CONCEPT_USAGE(IncidenceGraph) {
+          BOOST_CONCEPT_ASSERT((MultiPassInputIterator<out_edge_iterator>));
+          BOOST_CONCEPT_ASSERT((DefaultConstructible<edge_descriptor>));
+          BOOST_CONCEPT_ASSERT((EqualityComparable<edge_descriptor>));
+          BOOST_CONCEPT_ASSERT((Assignable<edge_descriptor>));
+          BOOST_CONCEPT_ASSERT((Convertible<traversal_category,
+                                incidence_graph_tag>));
+
+          p = out_edges(u, g);
+          n = out_degree(u, g);
+          e = *p.first;
+          u = source(e, g);
+          v = target(e, g);
+          const_constraints(g);
+      }
+      void const_constraints(const G& cg) {
+          p = out_edges(u, cg);
+          n = out_degree(u, cg);
+          e = *p.first;
+          u = source(e, cg);
+          v = target(e, cg);
+      }
+      std::pair<out_edge_iterator, out_edge_iterator> p;
+      typename graph_traits<G>::vertex_descriptor u, v;
+      typename graph_traits<G>::edge_descriptor e;
+      typename graph_traits<G>::degree_size_type n;
+      G g;
+  };
+
+  BOOST_concept(BidirectionalGraph,(G))
+    : IncidenceGraph<G>
+  {
+    typedef typename graph_traits<G>::in_edge_iterator
+      in_edge_iterator;
+    typedef typename graph_traits<G>::traversal_category
+      traversal_category;
+
+    BOOST_CONCEPT_USAGE(BidirectionalGraph) {
+      BOOST_CONCEPT_ASSERT((MultiPassInputIterator<in_edge_iterator>));
+      BOOST_CONCEPT_ASSERT((Convertible<traversal_category,
+        bidirectional_graph_tag>));
+
+      p = in_edges(v, g);
+      n = in_degree(v, g);
+      e = *p.first;
+      const_constraints(g);
+    }
+    void const_constraints(const G& cg) {
+      p = in_edges(v, cg);
+      n = in_degree(v, cg);
+      e = *p.first;
+    }
+    std::pair<in_edge_iterator, in_edge_iterator> p;
+    typename graph_traits<G>::vertex_descriptor v;
+    typename graph_traits<G>::edge_descriptor e;
+    typename graph_traits<G>::degree_size_type n;
+    G g;
+  };
+
+  BOOST_concept(AdjacencyGraph,(G))
+    : Graph<G>
+  {
+    typedef typename graph_traits<G>::adjacency_iterator
+      adjacency_iterator;
+    typedef typename graph_traits<G>::traversal_category
+      traversal_category;
+
+    BOOST_CONCEPT_USAGE(AdjacencyGraph) {
+      BOOST_CONCEPT_ASSERT((MultiPassInputIterator<adjacency_iterator>));
+      BOOST_CONCEPT_ASSERT((Convertible<traversal_category,
+        adjacency_graph_tag>));
+
+      p = adjacent_vertices(v, g);
+      v = *p.first;
+      const_constraints(g);
+    }
+    void const_constraints(const G& cg) {
+      p = adjacent_vertices(v, cg);
+    }
+    std::pair<adjacency_iterator,adjacency_iterator> p;
+    typename graph_traits<G>::vertex_descriptor v;
+    G g;
+  };
+
+  BOOST_concept(VertexListGraph,(G))
+    : Graph<G>
   {
     typedef typename graph_traits<G>::vertex_iterator vertex_iterator;
     typedef typename graph_traits<G>::vertices_size_type vertices_size_type;
     typedef typename graph_traits<G>::traversal_category
       traversal_category;
-    void constraints() {
-      function_requires< GraphConcept<G> >();
-      function_requires< MultiPassInputIteratorConcept<vertex_iterator> >();
-      function_requires< ConvertibleConcept<traversal_category,
-        vertex_list_graph_tag> >();
+
+    BOOST_CONCEPT_USAGE(VertexListGraph) {
+      BOOST_CONCEPT_ASSERT((MultiPassInputIterator<vertex_iterator>));
+      BOOST_CONCEPT_ASSERT((Convertible<traversal_category,
+        vertex_list_graph_tag>));
 
 #ifdef BOOST_VECTOR_AS_GRAPH_GRAPH_ADL_HACK
-      // dwa 2003/7/11 -- This clearly shouldn't be neccessary, but if
+      // dwa 2003/7/11 -- This clearly shouldn't be necessary, but if
       // you want to use vector_as_graph, it is!  I'm sure the graph
       // library leaves these out all over the place.  Probably a
       // redesign involving specializing a template with a static
@@ -197,9 +186,9 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
       v = *p.first;
       const_constraints(g);
     }
-    void const_constraints(const G& g) {
+    void const_constraints(const G& cg) {
 #ifdef BOOST_VECTOR_AS_GRAPH_GRAPH_ADL_HACK
-      // dwa 2003/7/11 -- This clearly shouldn't be neccessary, but if
+      // dwa 2003/7/11 -- This clearly shouldn't be necessary, but if
       // you want to use vector_as_graph, it is!  I'm sure the graph
       // library leaves these out all over the place.  Probably a
       // redesign involving specializing a template with a static
@@ -207,9 +196,9 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
       using boost::vertices;
 #endif 
       
-      p = vertices(g);
+      p = vertices(cg);
       v = *p.first;
-      V = num_vertices(g);
+      V = num_vertices(cg);
     }
     std::pair<vertex_iterator,vertex_iterator> p;
     typename graph_traits<G>::vertex_descriptor v;
@@ -217,22 +206,22 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     vertices_size_type V;
   };
 
-  template <class G>
-  struct EdgeListGraphConcept
+  BOOST_concept(EdgeListGraph,(G))
+    : Graph<G>
   {
     typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
     typedef typename graph_traits<G>::edge_iterator edge_iterator;
     typedef typename graph_traits<G>::edges_size_type edges_size_type;
     typedef typename graph_traits<G>::traversal_category
       traversal_category;
-    void constraints() {
-      function_requires< GraphConcept<G> >();
-      function_requires< MultiPassInputIteratorConcept<edge_iterator> >();
-      function_requires< DefaultConstructibleConcept<edge_descriptor> >();
-      function_requires< EqualityComparableConcept<edge_descriptor> >();
-      function_requires< AssignableConcept<edge_descriptor> >();
-      function_requires< ConvertibleConcept<traversal_category,
-        edge_list_graph_tag> >();
+
+    BOOST_CONCEPT_USAGE(EdgeListGraph) {
+      BOOST_CONCEPT_ASSERT((MultiPassInputIterator<edge_iterator>));
+      BOOST_CONCEPT_ASSERT((DefaultConstructible<edge_descriptor>));
+      BOOST_CONCEPT_ASSERT((EqualityComparable<edge_descriptor>));
+      BOOST_CONCEPT_ASSERT((Assignable<edge_descriptor>));
+      BOOST_CONCEPT_ASSERT((Convertible<traversal_category,
+        edge_list_graph_tag>));
 
       p = edges(g);
       e = *p.first;
@@ -240,12 +229,12 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
       v = target(e, g);
       const_constraints(g);
     }
-    void const_constraints(const G& g) {
-      p = edges(g);
-      E = num_edges(g);
+    void const_constraints(const G& cg) {
+      p = edges(cg);
+      E = num_edges(cg);
       e = *p.first;
-      u = source(e, g);
-      v = target(e, g);
+      u = source(e, cg);
+      v = target(e, cg);
     }
     std::pair<edge_iterator,edge_iterator> p;
     typename graph_traits<G>::vertex_descriptor u, v;
@@ -254,13 +243,10 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     G g;
   };
 
-  template <class G>
-  struct VertexAndEdgeListGraphConcept
+  BOOST_concept(VertexAndEdgeListGraph,(G))
+    : VertexListGraph<G>
+    , EdgeListGraph<G>
   {
-    void constraints() {
-      function_requires< VertexListGraphConcept<G> >();    
-      function_requires< EdgeListGraphConcept<G> >();
-    }
   };
 
   // Where to put the requirement for this constructor?
@@ -268,11 +254,11 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
   // Not in mutable graph, then LEDA graph's can't be models of
   // MutableGraph.
 
-  template <class G>
-  struct EdgeMutableGraphConcept
+  BOOST_concept(EdgeMutableGraph,(G))
   {
     typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
-    void constraints() {
+
+    BOOST_CONCEPT_USAGE(EdgeMutableGraph) {
       p = add_edge(u, v, g);
       remove_edge(u, v, g);
       remove_edge(e, g);
@@ -284,10 +270,10 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename graph_traits<G>::vertex_descriptor u, v;
   };
 
-  template <class G>
-  struct VertexMutableGraphConcept
+  BOOST_concept(VertexMutableGraph,(G))
   {
-    void constraints() {
+
+      BOOST_CONCEPT_USAGE(VertexMutableGraph) {
       v = add_vertex(g);
       remove_vertex(v, g);
     }
@@ -295,13 +281,10 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename graph_traits<G>::vertex_descriptor u, v;
   };
 
-  template <class G>
-  struct MutableGraphConcept
+  BOOST_concept(MutableGraph,(G))
+    : EdgeMutableGraph<G>
+    , VertexMutableGraph<G>
   {
-    void constraints() {
-      function_requires< EdgeMutableGraphConcept<G> >();
-      function_requires< VertexMutableGraphConcept<G> >();
-    }
   };
 
   template <class edge_descriptor>
@@ -311,11 +294,10 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     }
   };
 
-  template <class G>
-  struct MutableIncidenceGraphConcept
+  BOOST_concept(MutableIncidenceGraph,(G))
+    : MutableGraph<G>
   {
-    void constraints() {
-      function_requires< MutableGraphConcept<G> >();
+    BOOST_CONCEPT_USAGE(MutableIncidenceGraph) {
       remove_edge(iter, g);
       remove_out_edge_if(u, p, g);
     }
@@ -326,24 +308,23 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename boost::graph_traits<G>::out_edge_iterator iter;
   };
 
-  template <class G>
-  struct MutableBidirectionalGraphConcept
+  BOOST_concept(MutableBidirectionalGraph,(G))
+    : MutableIncidenceGraph<G>
   {
-    void constraints() {
-      function_requires< MutableIncidenceGraphConcept<G> >();
-      remove_in_edge_if(u, p, g);
-    }
-    G g;
-    typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
-    dummy_edge_predicate<edge_descriptor> p;
-    typename boost::graph_traits<G>::vertex_descriptor u;
+      BOOST_CONCEPT_USAGE(MutableBidirectionalGraph)
+      {
+          remove_in_edge_if(u, p, g);
+      }
+      G g;
+      typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
+      dummy_edge_predicate<edge_descriptor> p;
+      typename boost::graph_traits<G>::vertex_descriptor u;
   };
 
-  template <class G>
-  struct MutableEdgeListGraphConcept
+  BOOST_concept(MutableEdgeListGraph,(G))
+    : EdgeMutableGraph<G>
   {
-    void constraints() {
-      function_requires< EdgeMutableGraphConcept<G> >();
+    BOOST_CONCEPT_USAGE(MutableEdgeListGraph) {
       remove_edge_if(p, g);
     }
     G g;
@@ -351,11 +332,10 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     dummy_edge_predicate<edge_descriptor> p;
   };
 
-  template <class G>
-  struct VertexMutablePropertyGraphConcept
+  BOOST_concept(VertexMutablePropertyGraph,(G))
+    : VertexMutableGraph<G>
   {
-    void constraints() {
-      function_requires< VertexMutableGraphConcept<G> >();
+    BOOST_CONCEPT_USAGE(VertexMutablePropertyGraph) {
       v = add_vertex(vp, g);
     }
     G g;
@@ -363,12 +343,12 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename vertex_property<G>::type vp;
   };
 
-  template <class G>
-  struct EdgeMutablePropertyGraphConcept
+  BOOST_concept(EdgeMutablePropertyGraph,(G))
+    : EdgeMutableGraph<G>
   {
     typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
-    void constraints() {
-      function_requires< EdgeMutableGraphConcept<G> >();
+
+    BOOST_CONCEPT_USAGE(EdgeMutablePropertyGraph) {
       p = add_edge(u, v, ep, g);
     }
     G g;
@@ -377,37 +357,37 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename edge_property<G>::type ep;
   };
 
-  template <class G>
-  struct AdjacencyMatrixConcept
+  BOOST_concept(AdjacencyMatrix,(G))
+    : Graph<G>
   {
     typedef typename graph_traits<G>::edge_descriptor edge_descriptor;
-    void constraints() {
-      function_requires< GraphConcept<G> >();
-      
+
+    BOOST_CONCEPT_USAGE(AdjacencyMatrix) {      
       p = edge(u, v, g);
       const_constraints(g);
     }
-    void const_constraints(const G& g) {
-      p = edge(u, v, g);
+    void const_constraints(const G& cg) {
+      p = edge(u, v, cg);
     }
     typename graph_traits<G>::vertex_descriptor u, v;
     std::pair<edge_descriptor, bool> p;
     G g;
   };
 
-  template <class G, class X, class Property>
-  struct ReadablePropertyGraphConcept
+  BOOST_concept(ReadablePropertyGraph,(G)(X)(Property))
+    : Graph<G>
   {
     typedef typename property_map<G, Property>::const_type const_Map;
-    void constraints() {
-      function_requires< GraphConcept<G> >();
-      function_requires< ReadablePropertyMapConcept<const_Map, X> >();
+    
+    BOOST_CONCEPT_USAGE(ReadablePropertyGraph)
+    {
+      BOOST_CONCEPT_ASSERT((ReadablePropertyMapConcept<const_Map, X>));
 
       const_constraints(g);
     }
-    void const_constraints(const G& g) {
-      const_Map pmap = get(Property(), g);
-      pval = get(Property(), g, x);
+    void const_constraints(const G& cg) {
+      const_Map pmap = get(Property(), cg);
+      pval = get(Property(), cg, x);
       ignore_unused_variable_warning(pmap);
     }
     G g;
@@ -415,13 +395,12 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename property_traits<const_Map>::value_type pval;
   };
 
-  template <class G, class X, class Property>
-  struct PropertyGraphConcept
+  BOOST_concept(PropertyGraph,(G)(X)(Property))
+    : ReadablePropertyGraph<G, X, Property>
   {
     typedef typename property_map<G, Property>::type Map;
-    void constraints() {
-      function_requires< ReadablePropertyGraphConcept<G, X, Property> >();
-      function_requires< ReadWritePropertyMapConcept<Map, X> >();
+    BOOST_CONCEPT_USAGE(PropertyGraph) {
+      BOOST_CONCEPT_ASSERT((ReadWritePropertyMapConcept<Map, X>));
 
       Map pmap = get(Property(), g);
       pval = get(Property(), g, x);
@@ -433,14 +412,14 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     typename property_traits<Map>::value_type pval;
   };
 
-  template <class G, class X, class Property>
-  struct LvaluePropertyGraphConcept
+  BOOST_concept(LvaluePropertyGraph,(G)(X)(Property))
+    : ReadablePropertyGraph<G, X, Property>
   {
     typedef typename property_map<G, Property>::type Map;
     typedef typename property_map<G, Property>::const_type const_Map;
-    void constraints() {
-      function_requires< ReadablePropertyGraphConcept<G, X, Property> >();
-      function_requires< LvaluePropertyMapConcept<const_Map, X> >();
+
+    BOOST_CONCEPT_USAGE(LvaluePropertyGraph) {
+      BOOST_CONCEPT_ASSERT((LvaluePropertyMapConcept<const_Map, X>));
 
       pval = get(Property(), g, x);
       put(Property(), g, x, pval);
@@ -451,20 +430,19 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
   };
 
   // This needs to move out of the graph library
-  template <class B>
-  struct BufferConcept
+  BOOST_concept(Buffer,(B))
   {
-    void constraints() {
+    BOOST_CONCEPT_USAGE(Buffer) {
       b.push(t);
       b.pop();
       typename B::value_type& v = b.top();
       const_constraints(b);
       ignore_unused_variable_warning(v);
     }
-    void const_constraints(const B& b) {
-      const typename B::value_type& v = b.top();
-      n = b.size();
-      bool e = b.empty();
+    void const_constraints(const B& cb) {
+      const typename B::value_type& v = cb.top();
+      n = cb.size();
+      bool e = cb.empty();
       ignore_unused_variable_warning(v);
       ignore_unused_variable_warning(e);
     }
@@ -473,13 +451,11 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     B b;
   };
 
-  template <class C>
-  struct ColorValueConcept
+  BOOST_concept(ColorValue,(C))
+    : EqualityComparable<C>
+    , DefaultConstructible<C>
   {
-    void constraints() {
-      function_requires< EqualityComparableConcept<C> >();
-      function_requires< DefaultConstructibleConcept<C> >();
-
+    BOOST_CONCEPT_USAGE(ColorValue) {
       c = color_traits<C>::white();
       c = color_traits<C>::gray();
       c = color_traits<C>::black();
@@ -487,22 +463,48 @@ typename T::ThereReallyIsNoMemberByThisNameInT vertices(T const&);
     C c;
   };
 
-  template <class M, class I, class V>
-  struct BasicMatrixConcept
+  BOOST_concept(BasicMatrix,(M)(I)(V))
   {
-    void constraints() {
+    BOOST_CONCEPT_USAGE(BasicMatrix) {
       V& elt = A[i][j];
       const_constraints(A);
       ignore_unused_variable_warning(elt);      
     }
-    void const_constraints(const M& A) {
-      const V& elt = A[i][j];
+    void const_constraints(const M& cA) {
+      const V& elt = cA[i][j];
       ignore_unused_variable_warning(elt);      
     }
     M A;
     I i, j;
   };
 
+  } // end namespace concepts
+
+  using boost::concepts::MultiPassInputIteratorConcept;
+  using boost::concepts::GraphConcept;
+  using boost::concepts::IncidenceGraphConcept;
+  using boost::concepts::BidirectionalGraphConcept;
+  using boost::concepts::AdjacencyGraphConcept;
+  using boost::concepts::VertexListGraphConcept;
+  using boost::concepts::EdgeListGraphConcept;
+  using boost::concepts::VertexAndEdgeListGraphConcept;
+  using boost::concepts::EdgeMutableGraphConcept;
+  using boost::concepts::VertexMutableGraphConcept;
+  using boost::concepts::MutableGraphConcept;
+  using boost::concepts::MutableIncidenceGraphConcept;
+  using boost::concepts::MutableBidirectionalGraphConcept;
+  using boost::concepts::MutableEdgeListGraphConcept;
+  using boost::concepts::VertexMutablePropertyGraphConcept;
+  using boost::concepts::EdgeMutablePropertyGraphConcept;
+  using boost::concepts::AdjacencyMatrixConcept;
+  using boost::concepts::ReadablePropertyGraphConcept;
+  using boost::concepts::PropertyGraphConcept;
+  using boost::concepts::LvaluePropertyGraphConcept;
+  using boost::concepts::BufferConcept;
+  using boost::concepts::ColorValueConcept;
+  using boost::concepts::BasicMatrixConcept;
 } // namespace boost
 
+#include <boost/concept/detail/concept_undef.hpp>
+
 #endif /* BOOST_GRAPH_CONCEPTS_H */
